/* $Id$ */
// Copyright (C) 2002, International Business Machines
// Corporation and others.  All Rights Reserved.
// This code is licensed under the terms of the Eclipse Public License (EPL).

#ifndef CbcCompareActual_H
#define CbcCompareActual_H
#include "CbcNode.hpp"
#include "CbcCompareBase.hpp"
#include "CbcCompare.hpp"
#include "CbcCompareDepth.hpp"
#include "CbcCompareDefault.hpp"
#endif

/* vi: softtabstop=2 shiftwidth=2 expandtab tabstop=2
*/
